@@ -33,7 +33,11 @@
 #ifdef HAVE_INTTYPES_H
 #include <inttypes.h>
 #endif // HAVE_INTTYPES_H
-
+#include <sys/types.h>
+#include <sys/stat.h>
+#ifdef HAVE_FCNTL_H
+#include <fcntl.h>
+#endif // HAVE_FCNTL_H
 #include <sys/wait.h>
 
 #include <cerrno>
@@ -54,14 +58,14 @@ using namespace nghttp2;
 namespace shrpx {
 
 namespace {
-const StringRef SEVERITY_STR[] = {
+constexpr StringRef SEVERITY_STR[] = {
     StringRef::from_lit("INFO"), StringRef::from_lit("NOTICE"),
     StringRef::from_lit("WARN"), StringRef::from_lit("ERROR"),
     StringRef::from_lit("FATAL")};
 } // namespace
 
 namespace {
-const char *SEVERITY_COLOR[] = {
+constexpr const char *SEVERITY_COLOR[] = {
     "\033[1;32m", // INFO
     "\033[1;36m", // NOTICE
     "\033[1;33m", // WARN
@@ -142,7 +146,7 @@ Log::~Log() {
   // Error log format: <datetime> <master-pid> <current-pid>
   // <thread-id> <level> (<filename>:<line>) <msg>
   rv = snprintf(buf, sizeof(buf), "%s %d %d %s %s%s%s (%s:%d) %s\n",
-                lgconf->time_iso8601.c_str(), config->pid, lgconf->pid,
+                lgconf->tstamp->time_iso8601.c_str(), config->pid, lgconf->pid,
                 lgconf->thread_id.c_str(), tty ? SEVERITY_COLOR[severity_] : "",
                 SEVERITY_STR[severity_].c_str(), tty ? "\033[0m" : "",
                 filename_, linenum_, stream_.str().c_str());
@@ -159,74 +163,124 @@ Log::~Log() {
 
 namespace {
 template <typename OutputIterator>
-std::pair<OutputIterator, size_t> copy(const char *src, size_t srclen,
-                                       size_t avail, OutputIterator oitr) {
-  auto nwrite = std::min(srclen, avail);
-  auto noitr = std::copy_n(src, nwrite, oitr);
-  return std::make_pair(noitr, avail - nwrite);
+std::pair<OutputIterator, OutputIterator> copy(const char *src, size_t srclen,
+                                               OutputIterator d_first,
+                                               OutputIterator d_last) {
+  auto nwrite =
+      std::min(static_cast<size_t>(std::distance(d_first, d_last)), srclen);
+  return std::make_pair(std::copy_n(src, nwrite, d_first), d_last);
 }
 } // namespace
 
 namespace {
 template <typename OutputIterator>
-std::pair<OutputIterator, size_t> copy(const char *src, size_t avail,
-                                       OutputIterator oitr) {
-  return copy(src, strlen(src), avail, oitr);
+std::pair<OutputIterator, OutputIterator>
+copy(const char *src, OutputIterator d_first, OutputIterator d_last) {
+  return copy(src, strlen(src), d_first, d_last);
 }
 } // namespace
 
 namespace {
 template <typename OutputIterator>
-std::pair<OutputIterator, size_t> copy(const std::string &src, size_t avail,
-                                       OutputIterator oitr) {
-  return copy(src.c_str(), src.size(), avail, oitr);
-}
-} // namespace
-
-namespace {
-template <typename OutputIterator>
-std::pair<OutputIterator, size_t> copy(const StringRef &src, size_t avail,
-                                       OutputIterator oitr) {
-  return copy(src.c_str(), src.size(), avail, oitr);
-}
-} // namespace
-
-namespace {
-template <typename OutputIterator>
-std::pair<OutputIterator, size_t> copy(const ImmutableString &src, size_t avail,
-                                       OutputIterator oitr) {
-  return copy(src.c_str(), src.size(), avail, oitr);
+std::pair<OutputIterator, OutputIterator>
+copy(const StringRef &src, OutputIterator d_first, OutputIterator d_last) {
+  return copy(src.c_str(), src.size(), d_first, d_last);
 }
 } // namespace
 
 namespace {
 template <size_t N, typename OutputIterator>
-std::pair<OutputIterator, size_t> copy_l(const char (&src)[N], size_t avail,
-                                         OutputIterator oitr) {
-  return copy(src, N - 1, avail, oitr);
+std::pair<OutputIterator, OutputIterator>
+copy_l(const char (&src)[N], OutputIterator d_first, OutputIterator d_last) {
+  return copy(src, N - 1, d_first, d_last);
 }
 } // namespace
 
 namespace {
-const char LOWER_XDIGITS[] = "0123456789abcdef";
+template <typename OutputIterator>
+std::pair<OutputIterator, OutputIterator> copy(char c, OutputIterator d_first,
+                                               OutputIterator d_last) {
+  if (d_first == d_last) {
+    return std::make_pair(d_last, d_last);
+  }
+  *d_first++ = c;
+  return std::make_pair(d_first, d_last);
+}
+} // namespace
+
+namespace {
+constexpr char LOWER_XDIGITS[] = "0123456789abcdef";
 } // namespace
 
 namespace {
 template <typename OutputIterator>
-std::pair<OutputIterator, size_t> copy_hex_low(const uint8_t *src,
-                                               size_t srclen, size_t avail,
-                                               OutputIterator oitr) {
-  auto nwrite = std::min(srclen * 2, avail) / 2;
-  for (auto i = 0u; i < nwrite; ++i) {
-    *oitr++ = LOWER_XDIGITS[src[i] >> 4];
-    *oitr++ = LOWER_XDIGITS[src[i] & 0xf];
+std::pair<OutputIterator, OutputIterator>
+copy_hex_low(const uint8_t *src, size_t srclen, OutputIterator d_first,
+             OutputIterator d_last) {
+  auto nwrite = std::min(static_cast<size_t>(std::distance(d_first, d_last)),
+                         srclen * 2) /
+                2;
+  for (size_t i = 0; i < nwrite; ++i) {
+    *d_first++ = LOWER_XDIGITS[src[i] >> 4];
+    *d_first++ = LOWER_XDIGITS[src[i] & 0xf];
   }
-  return std::make_pair(oitr, avail - nwrite);
+  return std::make_pair(d_first, d_last);
+}
+} // namespace
+
+namespace {
+template <typename OutputIterator, typename T>
+std::pair<OutputIterator, OutputIterator> copy(T n, OutputIterator d_first,
+                                               OutputIterator d_last) {
+  if (static_cast<size_t>(std::distance(d_first, d_last)) <
+      NGHTTP2_MAX_UINT64_DIGITS) {
+    return std::make_pair(d_last, d_last);
+  }
+  return std::make_pair(util::utos(d_first, n), d_last);
+}
+} // namespace
+
+namespace {
+// Construct absolute request URI from |Request|, mainly to log
+// request URI for proxy request (HTTP/2 proxy or client proxy).  This
+// is mostly same routine found in
+// HttpDownstreamConnection::push_request_headers(), but vastly
+// simplified since we only care about absolute URI.
+StringRef construct_absolute_request_uri(BlockAllocator &balloc,
+                                         const Request &req) {
+  if (req.authority.empty()) {
+    return req.path;
+  }
+
+  auto len = req.authority.size() + req.path.size();
+  if (req.scheme.empty()) {
+    len += str_size("http://");
+  } else {
+    len += req.scheme.size() + str_size("://");
+  }
+
+  auto iov = make_byte_ref(balloc, len + 1);
+  auto p = iov.base;
+
+  if (req.scheme.empty()) {
+    // We may have to log the request which lacks scheme (e.g.,
+    // http/1.1 with origin form).
+    p = util::copy_lit(p, "http://");
+  } else {
+    p = std::copy(std::begin(req.scheme), std::end(req.scheme), p);
+    p = util::copy_lit(p, "://");
+  }
+  p = std::copy(std::begin(req.authority), std::end(req.authority), p);
+  p = std::copy(std::begin(req.path), std::end(req.path), p);
+  *p = '\0';
+
+  return StringRef{iov.base, p};
 }
 } // namespace
 
 void upstream_accesslog(const std::vector<LogFragment> &lfv,
                         const LogSpec &lgsp) {
+  auto config = get_config();
   auto lgconf = log_config();
   auto &accessconf = get_config()->logging.access;
 
@@ -234,148 +288,150 @@ void upstream_accesslog(const std::vector<LogFragment> &lfv,
     return;
   }
 
-  char buf[4_k];
+  std::array<char, 4_k> buf;
 
   auto downstream = lgsp.downstream;
 
-  const Request *req = nullptr;
-  if (downstream) {
-    req = &downstream->request();
-  }
+  const auto &req = downstream->request();
+  const auto &resp = downstream->response();
+  const auto &tstamp = req.tstamp;
+  auto &balloc = downstream->get_block_allocator();
 
-  auto p = buf;
-  auto avail = sizeof(buf) - 2;
+  auto downstream_addr = downstream->get_addr();
+  auto method = http2::to_method_string(req.method);
+  auto path = req.method == HTTP_CONNECT
+                  ? req.authority
+                  : config->http2_proxy
+                        ? construct_absolute_request_uri(balloc, req)
+                        : req.path.empty()
+                              ? req.method == HTTP_OPTIONS
+                                    ? StringRef::from_lit("*")
+                                    : StringRef::from_lit("-")
+                              : req.path;
 
-  lgconf->update_tstamp(lgsp.time_now);
-  auto &time_local = lgconf->time_local;
-  auto &time_iso8601 = lgconf->time_iso8601;
+  auto p = std::begin(buf);
+  auto last = std::end(buf) - 2;
 
   for (auto &lf : lfv) {
     switch (lf.type) {
     case SHRPX_LOGF_LITERAL:
-      std::tie(p, avail) = copy(lf.value, avail, p);
+      std::tie(p, last) = copy(lf.value, p, last);
       break;
     case SHRPX_LOGF_REMOTE_ADDR:
-      std::tie(p, avail) = copy(lgsp.remote_addr, avail, p);
+      std::tie(p, last) = copy(lgsp.remote_addr, p, last);
       break;
     case SHRPX_LOGF_TIME_LOCAL:
-      std::tie(p, avail) = copy(time_local, avail, p);
+      std::tie(p, last) = copy(tstamp->time_local, p, last);
       break;
     case SHRPX_LOGF_TIME_ISO8601:
-      std::tie(p, avail) = copy(time_iso8601, avail, p);
+      std::tie(p, last) = copy(tstamp->time_iso8601, p, last);
       break;
     case SHRPX_LOGF_REQUEST:
-      std::tie(p, avail) = copy(lgsp.method, avail, p);
-      std::tie(p, avail) = copy_l(" ", avail, p);
-      std::tie(p, avail) = copy(lgsp.path, avail, p);
-      std::tie(p, avail) = copy_l(" HTTP/", avail, p);
-      std::tie(p, avail) = copy(util::utos(lgsp.major), avail, p);
-      if (lgsp.major < 2) {
-        std::tie(p, avail) = copy_l(".", avail, p);
-        std::tie(p, avail) = copy(util::utos(lgsp.minor), avail, p);
+      std::tie(p, last) = copy(method, p, last);
+      std::tie(p, last) = copy(' ', p, last);
+      std::tie(p, last) = copy(path, p, last);
+      std::tie(p, last) = copy_l(" HTTP/", p, last);
+      std::tie(p, last) = copy(req.http_major, p, last);
+      if (req.http_major < 2) {
+        std::tie(p, last) = copy('.', p, last);
+        std::tie(p, last) = copy(req.http_minor, p, last);
       }
       break;
     case SHRPX_LOGF_STATUS:
-      std::tie(p, avail) = copy(util::utos(lgsp.status), avail, p);
+      std::tie(p, last) = copy(resp.http_status, p, last);
       break;
     case SHRPX_LOGF_BODY_BYTES_SENT:
-      std::tie(p, avail) = copy(util::utos(lgsp.body_bytes_sent), avail, p);
+      std::tie(p, last) = copy(downstream->response_sent_body_length, p, last);
       break;
-    case SHRPX_LOGF_HTTP:
-      if (req) {
-        auto hd = req->fs.header(lf.value);
-        if (hd) {
-          std::tie(p, avail) = copy((*hd).value, avail, p);
-          break;
-        }
+    case SHRPX_LOGF_HTTP: {
+      auto hd = req.fs.header(lf.value);
+      if (hd) {
+        std::tie(p, last) = copy((*hd).value, p, last);
+        break;
       }
 
-      std::tie(p, avail) = copy_l("-", avail, p);
+      std::tie(p, last) = copy('-', p, last);
 
       break;
+    }
     case SHRPX_LOGF_AUTHORITY:
-      if (req) {
-        if (!req->authority.empty()) {
-          std::tie(p, avail) = copy(req->authority, avail, p);
-          break;
-        }
+      if (!req.authority.empty()) {
+        std::tie(p, last) = copy(req.authority, p, last);
+        break;
       }
 
-      std::tie(p, avail) = copy_l("-", avail, p);
+      std::tie(p, last) = copy('-', p, last);
 
       break;
     case SHRPX_LOGF_REMOTE_PORT:
-      std::tie(p, avail) = copy(lgsp.remote_port, avail, p);
+      std::tie(p, last) = copy(lgsp.remote_port, p, last);
       break;
     case SHRPX_LOGF_SERVER_PORT:
-      std::tie(p, avail) = copy(util::utos(lgsp.server_port), avail, p);
+      std::tie(p, last) = copy(lgsp.server_port, p, last);
       break;
     case SHRPX_LOGF_REQUEST_TIME: {
       auto t = std::chrono::duration_cast<std::chrono::milliseconds>(
-                   lgsp.request_end_time - lgsp.request_start_time)
+                   lgsp.request_end_time - downstream->get_request_start_time())
                    .count();
-
-      auto frac = util::utos(t % 1000);
-      auto sec = util::utos(t / 1000);
-      if (frac.size() < 3) {
-        frac = std::string(3 - frac.size(), '0') + frac;
+      std::tie(p, last) = copy(t / 1000, p, last);
+      std::tie(p, last) = copy('.', p, last);
+      auto frac = t % 1000;
+      if (frac < 100) {
+        auto n = frac < 10 ? 2 : 1;
+        std::tie(p, last) = copy("000", n, p, last);
       }
-      sec += '.';
-      sec += frac;
-
-      std::tie(p, avail) = copy(sec, avail, p);
-    } break;
+      std::tie(p, last) = copy(frac, p, last);
+      break;
+    }
     case SHRPX_LOGF_PID:
-      std::tie(p, avail) = copy(util::utos(lgsp.pid), avail, p);
+      std::tie(p, last) = copy(lgsp.pid, p, last);
       break;
     case SHRPX_LOGF_ALPN:
-      std::tie(p, avail) = copy(lgsp.alpn, avail, p);
+      std::tie(p, last) = copy(lgsp.alpn, p, last);
       break;
     case SHRPX_LOGF_SSL_CIPHER:
       if (!lgsp.tls_info) {
-        std::tie(p, avail) = copy_l("-", avail, p);
+        std::tie(p, last) = copy('-', p, last);
         break;
       }
-      std::tie(p, avail) = copy(lgsp.tls_info->cipher, avail, p);
+      std::tie(p, last) = copy(lgsp.tls_info->cipher, p, last);
       break;
     case SHRPX_LOGF_SSL_PROTOCOL:
       if (!lgsp.tls_info) {
-        std::tie(p, avail) = copy_l("-", avail, p);
+        std::tie(p, last) = copy('-', p, last);
         break;
       }
-      std::tie(p, avail) = copy(lgsp.tls_info->protocol, avail, p);
+      std::tie(p, last) = copy(lgsp.tls_info->protocol, p, last);
       break;
     case SHRPX_LOGF_SSL_SESSION_ID:
       if (!lgsp.tls_info || lgsp.tls_info->session_id_length == 0) {
-        std::tie(p, avail) = copy_l("-", avail, p);
+        std::tie(p, last) = copy('-', p, last);
         break;
       }
-      std::tie(p, avail) =
-          copy_hex_low(lgsp.tls_info->session_id,
-                       lgsp.tls_info->session_id_length, avail, p);
+      std::tie(p, last) = copy_hex_low(
+          lgsp.tls_info->session_id, lgsp.tls_info->session_id_length, p, last);
       break;
     case SHRPX_LOGF_SSL_SESSION_REUSED:
       if (!lgsp.tls_info) {
-        std::tie(p, avail) = copy_l("-", avail, p);
+        std::tie(p, last) = copy('-', p, last);
         break;
       }
-      std::tie(p, avail) =
-          copy_l(lgsp.tls_info->session_reused ? "r" : ".", avail, p);
+      std::tie(p, last) =
+          copy(lgsp.tls_info->session_reused ? 'r' : '.', p, last);
       break;
     case SHRPX_LOGF_BACKEND_HOST:
-      if (!lgsp.downstream_addr) {
-        std::tie(p, avail) = copy_l("-", avail, p);
+      if (!downstream_addr) {
+        std::tie(p, last) = copy('-', p, last);
         break;
       }
-      std::tie(p, avail) = copy(lgsp.downstream_addr->host, avail, p);
+      std::tie(p, last) = copy(downstream_addr->host, p, last);
       break;
     case SHRPX_LOGF_BACKEND_PORT:
-      if (!lgsp.downstream_addr) {
-        std::tie(p, avail) = copy_l("-", avail, p);
+      if (!downstream_addr) {
+        std::tie(p, last) = copy('-', p, last);
         break;
       }
-      std::tie(p, avail) =
-          copy(util::utos(lgsp.downstream_addr->port), avail, p);
+      std::tie(p, last) = copy(downstream_addr->port, p, last);
       break;
     case SHRPX_LOGF_NONE:
       break;
@@ -387,15 +443,16 @@ void upstream_accesslog(const std::vector<LogFragment> &lfv,
   *p = '\0';
 
   if (accessconf.syslog) {
-    syslog(LOG_INFO, "%s", buf);
+    syslog(LOG_INFO, "%s", buf.data());
 
     return;
   }
 
   *p++ = '\n';
 
-  auto nwrite = p - buf;
-  while (write(lgconf->accesslog_fd, buf, nwrite) == -1 && errno == EINTR)
+  auto nwrite = std::distance(std::begin(buf), p);
+  while (write(lgconf->accesslog_fd, buf.data(), nwrite) == -1 &&
+         errno == EINTR)
     ;
 }
 
@@ -410,7 +467,7 @@ int reopen_log_files() {
   auto &errorconf = config->logging.error;
 
   if (!accessconf.syslog && !accessconf.file.empty()) {
-    new_accesslog_fd = util::open_log_file(accessconf.file.c_str());
+    new_accesslog_fd = open_log_file(accessconf.file.c_str());
 
     if (new_accesslog_fd == -1) {
       LOG(ERROR) << "Failed to open accesslog file " << accessconf.file;
@@ -419,7 +476,7 @@ int reopen_log_files() {
   }
 
   if (!errorconf.syslog && !errorconf.file.empty()) {
-    new_errorlog_fd = util::open_log_file(errorconf.file.c_str());
+    new_errorlog_fd = open_log_file(errorconf.file.c_str());
 
     if (new_errorlog_fd == -1) {
       if (lgconf->errorlog_fd != -1) {
@@ -433,8 +490,8 @@ int reopen_log_files() {
     }
   }
 
-  util::close_log_file(lgconf->accesslog_fd);
-  util::close_log_file(lgconf->errorlog_fd);
+  close_log_file(lgconf->accesslog_fd);
+  close_log_file(lgconf->errorlog_fd);
 
   lgconf->accesslog_fd = new_accesslog_fd;
   lgconf->errorlog_fd = new_errorlog_fd;
@@ -478,4 +535,60 @@ void redirect_stderr_to_errorlog() {
   dup2(lgconf->errorlog_fd, STDERR_FILENO);
 }
 
+namespace {
+int STDERR_COPY = -1;
+int STDOUT_COPY = -1;
+} // namespace
+
+void store_original_fds() {
+  // consider dup'ing stdout too
+  STDERR_COPY = dup(STDERR_FILENO);
+  STDOUT_COPY = STDOUT_FILENO;
+  // no race here, since it is called early
+  util::make_socket_closeonexec(STDERR_COPY);
+}
+
+void restore_original_fds() { dup2(STDERR_COPY, STDERR_FILENO); }
+
+void close_log_file(int &fd) {
+  if (fd != STDERR_COPY && fd != STDOUT_COPY && fd != -1) {
+    close(fd);
+  }
+  fd = -1;
+}
+
+int open_log_file(const char *path) {
+
+  if (strcmp(path, "/dev/stdout") == 0 ||
+      strcmp(path, "/proc/self/fd/1") == 0) {
+    return STDOUT_COPY;
+  }
+
+  if (strcmp(path, "/dev/stderr") == 0 ||
+      strcmp(path, "/proc/self/fd/2") == 0) {
+    return STDERR_COPY;
+  }
+#if defined O_CLOEXEC
+
+  auto fd = open(path, O_WRONLY | O_APPEND | O_CREAT | O_CLOEXEC,
+                 S_IRUSR | S_IWUSR | S_IRGRP);
+#else // !O_CLOEXEC
+
+  auto fd =
+      open(path, O_WRONLY | O_APPEND | O_CREAT, S_IRUSR | S_IWUSR | S_IRGRP);
+
+  // We get race condition if execve is called at the same time.
+  if (fd != -1) {
+    util::make_socket_closeonexec(fd);
+  }
+
+#endif // !O_CLOEXEC
+
+  if (fd == -1) {
+    return -1;
+  }
+
+  return fd;
+}
+
 } // namespace shrpx
