@@ -35,8 +35,19 @@ using namespace nghttp2;
 
 namespace shrpx {
 
+Timestamp::Timestamp(const std::chrono::system_clock::time_point &tp) {
+  time_local = util::format_common_log(time_local_buf.data(), tp);
+  time_iso8601 = util::format_iso8601(time_iso8601_buf.data(), tp);
+  time_http = util::format_http_date(time_http_buf.data(), tp);
+}
+
 LogConfig::LogConfig()
-    : pid(getpid()), accesslog_fd(-1), errorlog_fd(-1), errorlog_tty(false) {
+    : time_str_updated(std::chrono::system_clock::now()),
+      tstamp(std::make_shared<Timestamp>(time_str_updated)),
+      pid(getpid()),
+      accesslog_fd(-1),
+      errorlog_fd(-1),
+      errorlog_tty(false) {
   auto tid = std::this_thread::get_id();
   auto tid_hash =
       util::hash32(StringRef{reinterpret_cast<uint8_t *>(&tid),
@@ -87,17 +98,15 @@ void delete_log_config() {}
 
 void LogConfig::update_tstamp(
     const std::chrono::system_clock::time_point &now) {
-  auto t0 = std::chrono::system_clock::to_time_t(time_str_updated_);
+  auto t0 = std::chrono::system_clock::to_time_t(time_str_updated);
   auto t1 = std::chrono::system_clock::to_time_t(now);
   if (t0 == t1) {
     return;
   }
 
-  time_str_updated_ = now;
+  time_str_updated = now;
 
-  time_local = util::format_common_log(time_local_buf.data(), now);
-  time_iso8601 = util::format_iso8601(time_iso8601_buf.data(), now);
-  time_http = util::format_http_date(time_http_buf.data(), now);
+  tstamp = std::make_shared<Timestamp>(now);
 }
 
 } // namespace shrpx
