@@ -37,15 +37,21 @@ using namespace nghttp2;
 
 namespace shrpx {
 
-struct LogConfig {
-  std::chrono::system_clock::time_point time_str_updated_;
+struct Timestamp {
+  Timestamp(const std::chrono::system_clock::time_point &tp);
+
   std::array<char, sizeof("03/Jul/2014:00:19:38 +0900")> time_local_buf;
   std::array<char, sizeof("2014-11-15T12:58:24.741+09:00")> time_iso8601_buf;
   std::array<char, sizeof("Mon, 10 Oct 2016 10:25:58 GMT")> time_http_buf;
-  std::string thread_id;
   StringRef time_local;
   StringRef time_iso8601;
   StringRef time_http;
+};
+
+struct LogConfig {
+  std::chrono::system_clock::time_point time_str_updated;
+  std::shared_ptr<Timestamp> tstamp;
+  std::string thread_id;
   pid_t pid;
   int accesslog_fd;
   int errorlog_fd;
