@@ -48,6 +48,8 @@
 #include "shrpx_ssl.h"
 #include "shrpx_live_check.h"
 #include "shrpx_connect_blocker.h"
+#include "shrpx_dns_tracker.h"
+#include "allocator.h"
 
 using namespace nghttp2;
 
@@ -75,15 +77,15 @@ struct DownstreamAddr {
   Address addr;
   // backend address.  If |host_unix| is true, this is UNIX domain
   // socket path.
-  ImmutableString host;
-  ImmutableString hostport;
+  StringRef host;
+  StringRef hostport;
   // backend port.  0 if |host_unix| is true.
   uint16_t port;
   // true if |host| contains UNIX domain socket path.
   bool host_unix;
 
   // sni field to send remote server if TLS is enabled.
-  ImmutableString sni;
+  StringRef sni;
 
   std::unique_ptr<ConnectBlocker> connect_blocker;
   std::unique_ptr<LiveCheck> live_check;
@@ -111,6 +113,8 @@ struct DownstreamAddr {
   shrpx_proto proto;
   // true if TLS is used in this backend
   bool tls;
+  // true if dynamic DNS is enabled
+  bool dns;
 };
 
 // Simplified weighted fair queuing.  Actually we don't use queue here
@@ -128,8 +132,18 @@ struct WeightedPri {
 
 struct SharedDownstreamAddr {
   SharedDownstreamAddr()
-      : next{0}, http1_pri{}, http2_pri{}, affinity{AFFINITY_NONE} {}
+      : balloc(1024, 1024),
+        next{0},
+        http1_pri{},
+        http2_pri{},
+        affinity{AFFINITY_NONE} {}
 
+  SharedDownstreamAddr(const SharedDownstreamAddr &) = delete;
+  SharedDownstreamAddr(SharedDownstreamAddr &&) = delete;
+  SharedDownstreamAddr &operator=(const SharedDownstreamAddr &) = delete;
+  SharedDownstreamAddr &operator=(SharedDownstreamAddr &&) = delete;
+
+  BlockAllocator balloc;
   std::vector<DownstreamAddr> addrs;
   // Bunch of session affinity hash.  Only used if affinity ==
   // AFFINITY_IP.
@@ -162,6 +176,11 @@ struct SharedDownstreamAddr {
 struct DownstreamAddrGroup {
   DownstreamAddrGroup() : retired{false} {};
 
+  DownstreamAddrGroup(const DownstreamAddrGroup &) = delete;
+  DownstreamAddrGroup(DownstreamAddrGroup &&) = delete;
+  DownstreamAddrGroup &operator=(const DownstreamAddrGroup &) = delete;
+  DownstreamAddrGroup &operator=(DownstreamAddrGroup &&) = delete;
+
   ImmutableString pattern;
   std::shared_ptr<SharedDownstreamAddr> shared_addr;
   // true if this group is no longer used for new request.  If this is
@@ -247,6 +266,8 @@ public:
 
   ConnectionHandler *get_connection_handler() const;
 
+  DNSTracker *get_dns_tracker();
+
 private:
 #ifndef NOTHREADS
   std::future<void> fut_;
@@ -259,6 +280,7 @@ private:
   ev_timer proc_wev_timer_;
   MemchunkPool mcpool_;
   WorkerStat worker_stat_;
+  DNSTracker dns_tracker_;
 
   std::shared_ptr<DownstreamConfig> downstreamconf_;
   std::unique_ptr<MemcachedDispatcher> session_cache_memcached_dispatcher_;
@@ -298,7 +320,10 @@ size_t match_downstream_addr_group(
     const std::vector<std::shared_ptr<DownstreamAddrGroup>> &groups,
     size_t catch_all, BlockAllocator &balloc);
 
-void downstream_failure(DownstreamAddr *addr);
+// Calls this function if connecting to backend failed.  |raddr| is
+// the actual address used to connect to backend, and it could be
+// nullptr.  This function may schedule live check.
+void downstream_failure(DownstreamAddr *addr, const Address *raddr);
 
 } // namespace shrpx
 
